//---------------------------------------------------//
//  UECS - Measurement Amount of Dischage 
//         Liquid and Fertilizer Node (LU-MADLF##)
//
// UARDECS_MEGAを使用して、
// Arduino mega 2560 + Arduino Ethernet 2 専用です
//
//  Version 1.0 2020.9.24 Progammed by T.Hoshi
//  Version 1.1 2020.10.1 自動排出状態でリセットしたとき
//                        自動排出終了時刻の設定ミスを修正
//  Version 1.2 2020.10.5 系統２の排肥量の日界時の計算ミス
//                        修正
//  Version 1.3 2020.10.19 排液温度CCM出力
//  Version 1.4 2020.11.30 3DP製造センサ用EC測定法とme
//                         計算方法の改良
//  Version 1.5 2020.12.04 超音波液量センサの計測値の変動
//                         の低下方向の抑止処理を加えた
//  Version 1.6 2021.08.20 セル定数とEC計測の小さなプログ
//                         ラムミスの修正
//  Version 1.7 2021.12.24 定刻排出の日界を超えるときの判
//                         断ミスの修正
//---------------------------------------------------//
#define verNO   "LU-MADLF17" // 10 char.
#define verDATE "20211224" // 8 char.
/*
-----------------------------------------------------------------------------
LU-MADLF## software are distributed under the 2 clause BSD license.

Copyright 2021, Takehiko Hoshi All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.

-----------------------------------------------------------------------------
LU-MADLF##ソフトウェアは、2条項BSDライセンスの下に配布されます。

Copyright 2021, Takehiko Hoshi All rights reserved.

  ソースコード形式かバイナリ形式か、変更するかしないかを問わず、以下の条件を
満たす場合に限り、再頒布および使用が許可されます。

1.ソースコードを再頒布する場合、上記の著作権表示、本条件一覧、および下記免責
  条項を含めること。
2.バイナリ形式で再頒布する場合、頒布物に付属のドキュメント等の資料に、上記の
  著作権表示、本条件一覧、および下記免責条項を含めること。

  本ソフトウェアは、著作権者およびコントリビューターによって「現状のまま」提供
されており、明示黙示を問わず、商業的な使用可能性、および特定の目的に対する適合
性に関する暗黙の保証も含め、またそれに限定されない、いかなる保証もありません。
著作権者もコントリビューターも、事由のいかんを問わず、 損害発生の原因いかんを
問わず、かつ責任の根拠が契約であるか厳格責任であるか（過失その他の）不法行為で
あるかを問わず、仮にそのような損害が発生する可能性を知らされていたとしても、
本ソフトウェアの使用によって発生した （代替品または代用サービスの調達、使用の
喪失、データの喪失、利益の喪失、業務の中断も含め、またそれに限定されない）直接
損害、間接損害、偶発的な損害、特別損害、懲罰的損害、または結果損害について、
一切責任を負わないものとします。
-----------------------------------------------------------------------------
*/
// このプログラムは、無償提供する代わりに、動作結果に関する
// 一切の保証は致しません。自己責任でお使いください。また、
// 改造も自由ですが、公刊等をされる場合は、「近畿大学　星　
// 岳彦が開発した○○〇プログラムを用いた」と引用して下さい
// ますと、今後の研究開発の励みになりますので、どうぞよろ
// しくお願いします。
//
 
// このシールドはArduino MEGA 2560 + Ethernet Shiedld2の
// 組み合わせで動作させることを前提にしています。
// Ethernet Sheild 2基板のシールのmacアドレスに変更して
// ください。このプログラムをmac[で検索すると付近にあり
// ます。macアドレスとは。90:A2:DA:0F:xx:xx のような8個
// の16進数です。これをばらして下の0xの後に転記してください。
#define mac0 0x90
#define mac1 0xA2
#define mac2 0xDA
#define mac3 0x11
#define mac4 0x37
#define mac5 0x5F

// UECS用のライブラリUARDECSのインストール
// https://uecs.org/arduino/uardecs.html が必要です。
#include <Uardecs_mega.h>
#include <Wire.h>
#include <SPI.h>
#include <Ethernet2.h>
#include <avr/pgmspace.h>
#include <EEPROM.h>

// I2Cのアドレス設定
#define LCD_ADRS  0x3E //液晶表示器
#define RTC_ADRS  0x32 //リアルタイムクロック

// 押しボタンのピン番号
#define PB_p 2
// 排出電磁弁のピン番号
#define DisChg_p 5
// 表示モード変数と押しボタンフラグ
int dispMode, swFlag ;

// HC-SR04とサーミスタのピン接続設定
int e_p[2] = { 7, 9 } ; // echo in
int t_p[2] = { 6, 8 } ; // trigger out
int TH_p[2] = { 0, 1 } ; // temperature
// EC測定ピン設定
int EC_Vout[2][2] = { {22, 11}, {12, 13} }  ;
int EC_Vin[2][2] = { {2, 3}, {4, 5} } ;
// EC測定時間設定(ms)
#define Tsetup 0 // 電圧をかけてから測定開始まで
// EC測定回路定数
#define constR 3300 // 負荷抵抗値　3.3 kΩ
#define constV 5000 // 印加電圧　5000 mV

//時刻用変数
byte secondS,minuteS,pminuteS,hourS,dayS,months,years;
signed long v_date ; //日付
signed long v_time ; //時刻

//計算用変数
float rowDist[2][65] ; // 距離を一次的に整列しておく配列
                       // 60秒だが時間ずれの余裕をみて各65個用意する
float distance[2], Cdist[2] ; //距離
float temperature[2] ; // 温度
float volume[2], Cvol[2]  ; // 排液量
float preVolume[2] ; // ECによる排肥量計算のための前回排液量
float voltEC[2], condEC[2] ; // ECセンサの電圧、導電率
int nofMD ; //　CCM送出サイクル中に計測できた回数
signed long dischgEndTime ; // 自動排出終了時刻

//作業用変数
int i, j, k, ch, wait ;

//LCD表示用バッファ他
char lcd_buf[20] ;

//////////////////////////////////
// UARDECS メイン設定
//////////////////////////////////

// UARDECS用IP初期化　ショートピン(D3)を抜くとセーフモード(192.168.1.7)
const byte U_InitPin = 3; // digital D3 pin for Safe mode
const byte U_InitPin_Sense=HIGH;

// UARDECSのUECSノード情報設定
const char U_name[] PROGMEM = verNO;
const char U_vender[] PROGMEM = "Hoshi-lab.";
const char U_uecsid[] PROGMEM = "01100900000F";
const char U_footnote[] PROGMEM = "Product on UARDECS121 by <A HREF=\"http://hoshi-lab.info/\">Kindai, hoshi-lab.</A>";
char U_nodename[20] = verNO;
UECSOriginalAttribute U_orgAttribute;

//////////////////////////////////
// html page setting
//////////////////////////////////

//Define variables for Web interface
signed long W_Mode ; // 表示・書込モード
signed long W_S1_V ; // センサ1測定値
signed long W_S2_V ; // センサ2測定値
signed long W_S1_EmpDis ; // センサ1空距離
signed long W_S2_EmpDis ; // センサ2空距離
signed long W_Adj1_Vol ; // 校正1の基準水量
signed long W_S1Adj1_Dis ; // センサ1の校正1での距離
signed long W_S2Adj1_Dis ; // センサ2の校正1での距離
signed long W_Adj2_Vol ; // 校正2の基準水量
signed long W_S1Adj2_Dis ; // センサ1の校正2での距離
signed long W_S2Adj2_Dis ; // センサ2の校正2での距離
signed long W_CellK1 ; // ECセンサ1のセル定数
signed long W_CellK2 ; // ECセンサ2のセル定数
signed long W_ECadj_Mode ; // ECセンサの校正　0 セル定数 1 EC値で
signed long W_time ; // 現在時刻
signed long W_timeSetup_Mode ; // 時刻設定モード　0 しない 1 する
signed long W_dischg_Time ; // 排出時刻
signed long W_dischg_Mode ; // 排出モード　0　停止　1　定刻　2 強制
signed long W_dischg_min ; // 排出時間　分間

// Define Strings
const char S_mode[] PROGMEM = "測定動作" ;
const char S_S1[] PROGMEM = "センサ1" ;
const char S_S2[] PROGMEM = "センサ2" ;
const char S_S1_EmpDis[] PROGMEM = "空距離1" ;
const char S_S2_EmpDis[] PROGMEM = "空距離2" ;
const char S_Adj1_vol[] PROGMEM = "校1液量" ;
const char S_S1Adj1_Dis[] PROGMEM = "校1距離1" ;
const char S_S2Adj1_Dis[] PROGMEM = "校1距離2" ;
const char S_Adj2_vol[] PROGMEM = "校2液量" ;
const char S_S1Adj2_Dis[] PROGMEM = "校2距離1" ;
const char S_S2Adj2_Dis[] PROGMEM = "校2距離2" ;
const char S_CellK1[] PROGMEM = "EC定数1" ;
const char S_CellK2[] PROGMEM = "EC定数2" ;
const char S_ECadj_Mode[] PROGMEM = "EC校正" ;
const char S_time[] PROGMEM = "現在時刻" ;
const char S_timeSetup_Mode[] PROGMEM = "時刻設定" ;
const char S_dischg_Time[] PROGMEM = "排出時刻" ;
const char S_dischg_Mode[] PROGMEM = "自動排出" ;
const char S_dischg_min[] PROGMEM = "排出時間" ;

//Select Strings
const char S_cc[] PROGMEM = "液量 cm3";
const char S_mm[] PROGMEM = "距離 mm";
const char S_at[] PROGMEM = "液温 x10℃";
const char S_vt[] PROGMEM = "電圧 mV";
const char S_ec[] PROGMEM = "EC x1k dS";
const char S_reset[] PROGMEM = "初期化";
const char *SS_mode[6]= { S_cc, S_mm, S_at, S_vt, S_ec, S_reset } ;
const char S_ts_no[] PROGMEM = "しない" ;
const char S_ts_yes[] PROGMEM = "する" ;
const char *SS_ts[2] = { S_ts_no, S_ts_yes } ;
const char S_ec_ck[] PROGMEM = "セル定数で" ;
const char S_ec_ec[] PROGMEM = "EC値で" ;
const char *SS_ec[2] = { S_ec_ck, S_ec_ec } ;
const char S_atdc_stop[] PROGMEM = "停止" ;
const char S_atdc_timer[] PROGMEM = "定刻" ;
const char S_atdc_fon[] PROGMEM = "強制" ;
const char *SS_atdc[3] = { S_atdc_stop, S_atdc_timer, S_atdc_fon } ;

//Unit Strings
const char U_cc[] PROGMEM = "cm3";
const char U_mm[] PROGMEM = "mm";
const char U_min[] PROGMEM = "分間" ;
const char U_hhmm[] PROGMEM = "時.分" ;

//Detail Strings
const char D_mode[] PROGMEM = "sendで測定" ;
const char D_meas[] PROGMEM = "測定値" ;
const char D_empt[] PROGMEM = "空の時" ;
const char D_a1v[] PROGMEM = "少量時" ;
const char D_a2v[] PROGMEM = "多量時" ;
const char D_ec[] PROGMEM = "/5000mV" ;
const char D_ECadj[] PROGMEM = "校正時入力" ;
const char D_disch_min[] PROGMEM = "排出弁開" ;
// Dummy Strings
const char EMPTY[] PROGMEM= "" ;
const char** DUMMY = NULL ;

const int U_HtmlLine = 19; //Total number of HTML table rows.

struct UECSUserHtml U_html[U_HtmlLine] = {
//{  名前,            入出力形式,       単位,   詳細説明,    選択肢文字列,選択肢数,    値,               最小値,最大値,小数桁数}
  { S_mode,          UECSSELECTDATA, EMPTY,  D_mode,     SS_mode,  6,        &(W_Mode),          0,  5,     0},
  { S_S1,            UECSSHOWDATA,   EMPTY,  D_meas,     DUMMY,    0,        &(W_S1_V),          0,  99999, 0},
  { S_S2,            UECSSHOWDATA,   EMPTY,  EMPTY,      DUMMY,    0,        &(W_S2_V),          0,  99999, 0},
  { S_S1_EmpDis,     UECSINPUTDATA,  U_mm,   D_empt,     DUMMY,    0,        &(W_S1_EmpDis),     0,  9999,  0},
  { S_S2_EmpDis,     UECSINPUTDATA,  U_mm,   EMPTY,      DUMMY,    0,        &(W_S2_EmpDis),     0,  9999,  0},
  { S_Adj1_vol,      UECSINPUTDATA,  U_cc,   D_a1v,      DUMMY,    0,        &(W_Adj1_Vol),      0,  99999, 0},
  { S_S1Adj1_Dis,    UECSINPUTDATA,  U_mm,   EMPTY,      DUMMY,    0,        &(W_S1Adj1_Dis),    0,  9999,  0},
  { S_S2Adj1_Dis,    UECSINPUTDATA,  U_mm,   EMPTY,      DUMMY,    0,        &(W_S2Adj1_Dis),    0,  9999,  0},
  { S_Adj2_vol,      UECSINPUTDATA,  U_cc,   D_a2v,      DUMMY,    0,        &(W_Adj2_Vol),      0,  99999, 0},
  { S_S1Adj2_Dis,    UECSINPUTDATA,  U_mm,   EMPTY,      DUMMY,    0,        &(W_S1Adj2_Dis),    0,  9999,  0},
  { S_S2Adj2_Dis,    UECSINPUTDATA,  U_mm,   EMPTY,      DUMMY,    0,        &(W_S2Adj2_Dis),    0,  9999,  0},
  { S_CellK1,        UECSINPUTDATA,  EMPTY,  D_ECadj,    DUMMY,    0,        &(W_CellK1),        0,  99999, 2},
  { S_CellK2,        UECSINPUTDATA,  EMPTY,  D_ECadj,    DUMMY,    0,        &(W_CellK2),        0,  99999, 2},
  { S_ECadj_Mode,    UECSSELECTDATA, EMPTY,  EMPTY,      SS_ec,    2,        &(W_ECadj_Mode)    ,0,  1,     0},
  { S_time,          UECSINPUTDATA,  U_hhmm, EMPTY,      DUMMY,    0,        &(W_time),          0,  2359,  2},
  { S_timeSetup_Mode,UECSSELECTDATA, EMPTY,  EMPTY,      SS_ts,    2,        &(W_timeSetup_Mode),0,  1,     0},
  { S_dischg_Mode,   UECSSELECTDATA, EMPTY,  EMPTY,      SS_atdc,  3,        &(W_dischg_Mode),   0,  2,     0},
  { S_dischg_Time,   UECSINPUTDATA,  U_hhmm, EMPTY,      DUMMY,    0,        &(W_dischg_Time),   0,  2359,  2},
  { S_dischg_min,    UECSINPUTDATA,  U_min,  D_disch_min,DUMMY,    0,        &(W_dischg_min),    1,  20,   0},
} ;

//UECS CCM 定義
enum {
CCMID_cnd,
CCMID_DLiquid1,
CCMID_DECVal1,
CCMID_DFert1,
CCMID_DNtemp1,
CCMID_DLiquid2,
CCMID_DECVal2,
CCMID_DFert2,
CCMID_DNtemp2,
CCMID_dummy
};

const int U_MAX_CCM = CCMID_dummy;
UECSCCM U_ccmList[U_MAX_CCM];

const char ccmNamecnd[] PROGMEM = "機器状態";
const char ccmNameDLiquidl[] PROGMEM = "排液量1";
const char ccmNameDLiquid2[] PROGMEM = "排液量2";
const char ccmNameDECVall[] PROGMEM = "排液EC1";
const char ccmNameDECVal2[] PROGMEM = "排液EC2";
const char ccmNameDFertl[] PROGMEM = "排肥量1";
const char ccmNameDFert2[] PROGMEM = "排肥量2";
const char ccmNameDNtempl[] PROGMEM = "排液温1";
const char ccmNameDNtemp2[] PROGMEM = "排液温2";

const char ccmTypecnd[] PROGMEM = "cnd.mHN";
const char ccmTypeDLiquid1[] PROGMEM = "DLiquid1.mHN";
const char ccmTypeDLiquid2[] PROGMEM = "DLiquid2.mHN";
const char ccmTypeDECVal1[] PROGMEM = "DECVal1.mHN";
const char ccmTypeDECVal2[] PROGMEM = "DECVal2.mHN";
const char ccmTypeDFert1[] PROGMEM = "DFert1.mHN";
const char ccmTypeDFert2[] PROGMEM = "DFert2.mHN";
const char ccmTypeDNtemp1[] PROGMEM = "DNtemp1.mHN";
const char ccmTypeDNtemp2[] PROGMEM = "DNtemp2.mHN";

const char ccmUnitcnd[] PROGMEM = "";
const char ccmUnitDLiquid[] PROGMEM = "cm3";
const char ccmUnitDECVal[] PROGMEM = "dS m-1";
const char ccmUnitDFert[] PROGMEM = "me";
const char ccmUnitDNtemp[] PROGMEM = "℃";

//------------------------------------------------------
//UARDECS初期化用関数
//主にCCMの作成とMACアドレスの設定を行う
//------------------------------------------------------
void UserInit(){
  //Set ccm list
  // sender, ID, Description, Type name, Unit, Priority, Decimal Digit, CCM Level
  UECSsetCCM(true, CCMID_cnd, ccmNamecnd, ccmTypecnd, ccmUnitcnd, 29, 0, A_1S_0) ;
  UECSsetCCM(true, CCMID_DLiquid1, ccmNameDLiquidl, ccmTypeDLiquid1, ccmUnitDLiquid, 29, 0, A_1M_0) ;
  UECSsetCCM(true, CCMID_DECVal1, ccmNameDECVall, ccmTypeDECVal1, ccmUnitDECVal, 29, 3, A_1M_0) ;
  UECSsetCCM(true, CCMID_DFert1, ccmNameDFertl, ccmTypeDFert1, ccmUnitDFert, 29, 1, A_1M_0) ;
  UECSsetCCM(true, CCMID_DNtemp1, ccmNameDNtempl, ccmTypeDNtemp1, ccmUnitDNtemp, 29, 1, A_1M_0) ;
  UECSsetCCM(true, CCMID_DLiquid2, ccmNameDLiquid2, ccmTypeDLiquid2, ccmUnitDLiquid, 29, 0, A_1M_0) ;
  UECSsetCCM(true, CCMID_DECVal2, ccmNameDECVal2, ccmTypeDECVal2, ccmUnitDECVal, 29, 3, A_1M_0) ;
  UECSsetCCM(true, CCMID_DFert2, ccmNameDFert2, ccmTypeDFert2, ccmUnitDFert, 29, 1, A_1M_0) ;
  UECSsetCCM(true, CCMID_DNtemp2, ccmNameDNtemp2, ccmTypeDNtemp2, ccmUnitDNtemp, 29, 1, A_1M_0) ;

  //必ずシールドに貼ってあるアドレスに書き換えて使用して下さい
  U_orgAttribute.mac[0] = mac0;
  U_orgAttribute.mac[1] = mac1;
  U_orgAttribute.mac[2] = mac2;
  U_orgAttribute.mac[3] = mac3;
  U_orgAttribute.mac[4] = mac4;
  U_orgAttribute.mac[5] = mac5;
}

//---------------------------------------------------------
//起動直後に１回呼び出される関数。
//様々な初期化処理を記述できる。
//この関数内ではUECSsetup()関数を呼び出さなくてはならない。
//必要に応じて処理を記述してもかまわない。
//---------------------------------------------------------
void setup() {

  // I2C初期化
  Wire.begin();

  //LCD初期化
  LCD_command(0x38);
  LCD_command(0x39);
  LCD_command(0x14);
  LCD_command(0x73);
  LCD_command(0x52);  // 5V
//LCD_command(0x56);  // 3V
  LCD_command(0x6C);
  LCD_command(0x38);
  LCD_command(0x01);
  LCD_command(0x0C);
  //LCDにタイトル表示
  LCD_command(0x80|0x00);
  String dispS = "** ";
  dispS.concat(verNO);
  dispS.concat(" **");
  dispS.toCharArray(lcd_buf, 17);
  LCD_string(lcd_buf) ;
  LCD_command(0x80|0x40);
  dispS = verDATE;
  dispS.concat(" T.Hoshi");
  dispS.toCharArray(lcd_buf, 17);
  LCD_string(lcd_buf) ;
  delay(2000) ;

  // RTC初期化
  Wire.beginTransmission(RTC_ADRS);
  Wire.write(0xE0);
  Wire.write(0x20);  //24hourMode
  Wire.write(0x00);  // PON Clear
  Wire.endTransmission();
  delay(1);
  RTCread() ; // 日付のみ初期化
  W_time = v_time ; // web時刻初期化
  v_date = 201130 ;
  RTCset() ;

  //押しボタン入力に
  pinMode(PB_p, INPUT_PULLUP) ;
  swFlag = 0 ; // 押しボタン押されたフラグクリア
  //表示を排液量
  dispMode = 0 ;

  //排出電磁弁のピンを出力にしてリセット
  pinMode(DisChg_p, OUTPUT) ;
  digitalWrite(DisChg_p, LOW) ;
  
  //距離センサ信号線の入出力設定
  for(i = 0; i < 2; i++) {
    pinMode(e_p[i], INPUT);
    pinMode(t_p[i], OUTPUT);
  }

  //ECセンサ信号線の入出力設定
  for(i = 0; i < 2; i++) {
    pinMode(EC_Vout[i][0], INPUT);
    pinMode(EC_Vout[i][1], INPUT);
  }

  UECSsetup();
  
  // W_mode変数の異常でリセットの条件判断
  if((W_Mode<0)||(W_Mode>5)) {
    initializeSetpoints();
  }
  // 自動排出終了時刻の仮設定 by ver.1.1
  dischgEndTime = W_dischg_Time + W_dischg_min ; // 自動排出終了時刻のセット
  if((dischgEndTime % 100) >= 60) dischgEndTime += 40 ; // 分桁上がり
  if(dischgEndTime >= 2400) dischgEndTime -= 2400 ; // 時桁上がり

  //CCM排肥料量の初期化
  U_ccmList[CCMID_DFert1].value = 0 ;
  U_ccmList[CCMID_DFert2].value = 0 ;

  nofMD = 0 ; // 計測サイクル内のカウンタをリセット
  preVolume[0] = 0.0 ;
  preVolume[1] = 0.0 ;
  measurementVol() ; // 初回排液量測定 
  MeasEC() ; // 初回EC測定
}

//------------------------------------------------------
// EPROM関係の制御設定値等をクリアする。
// W_modeの変数値がおかしい場合もリセット
//------------------------------------------------------
void initializeSetpoints() {
 W_Mode = 1 ; // 距離表示に
 W_S1_V = 0 ; // センサ1測定値
 W_S2_V = 0 ; // センサ2測定値
 W_S1_EmpDis = 300 ; // センサ1空距離
 W_S2_EmpDis = 300 ; // センサ2空距離
 W_Adj1_Vol = 3000 ; // 校正1の基準水量
 W_S1Adj1_Dis = 235  ; // センサ1の校正1での距離
 W_S2Adj1_Dis = 235 ; // センサ2の校正1での距離
 W_Adj2_Vol = 12000 ; // 校正2の基準水量
 W_S1Adj2_Dis = 60 ; // センサ1の校正2での距離
 W_S2Adj2_Dis = 60 ; // センサ2の校正2での距離
 W_CellK1 = 16000 ; // ECセル定数1
 W_CellK2 = 16000 ; // ECセル定数2
 W_ECadj_Mode = 0 ; // EC校正モードにしない
 W_time = 0 ;
 W_timeSetup_Mode = 0 ; // 時刻設定モードにしない
 W_dischg_Time = 0 ;
 W_dischg_Mode = 0 ;
 W_dischg_min = 1 ;

 
}

//---------------------------------------------------------
//Webページから入力が行われ各種値を設定し不揮発性メモリに値を保存後、
//ページの再描画を行う前に以下の関数が呼び出される。
//---------------------------------------------------------
void OnWebFormRecieved(){

  if(W_ECadj_Mode == 1) { // EC値で校正する場合の処理
    float adjEC1, adjEC2 ;
    adjEC1 = (float)W_CellK1 / 100.0 ;
    adjEC2 = (float)W_CellK2 / 100.0 ;
    // 仮のセル定数(160.00)に変更
    W_CellK1 = 16000 ;
    W_CellK2 = 16000 ;
    // 電圧測定
    MeasEC() ;
    // セル定数計算 100倍してW_CellK*に代入できる
    W_CellK1 = (adjEC1 * constR * voltEC[0]) / (constV - voltEC[0]) * 10.0 ;
    W_CellK1 /= 1 + 0.02 * (temperature[0] - 25) ;
    W_CellK2 = (adjEC2 * constR * voltEC[1]) / (constV - voltEC[1]) * 10.0 ;
    W_CellK2 /= 1 + 0.02 * (temperature[1] - 25) ;
    delay(1000) ; // 次にEC測定する場合に備えて再電圧印加まで時間を取る
    W_ECadj_Mode = 0 ;
  }

  switch(W_Mode) {  // modeの値により、測定・表示動作設定
    case 0: // 液量
      oneMeasVol() ; // 測定
      W_S1_V = (signed long)Cvol[0] ;
      W_S2_V = (signed long)Cvol[1] ;
      break ;
    case 1: // 距離
      oneMeasVol() ; // 測定
      W_S1_V = (signed long)Cdist[0] ;
      W_S2_V = (signed long)Cdist[1] ;
      break ;
    case 2: // 温度
      oneMeasVol() ; // 測定
      W_S1_V = (signed long)(temperature[0] * 10) ;
      W_S2_V = (signed long)(temperature[1] * 10) ;
      break ;
    case 3: // EC電極電圧
      MeasEC() ;
      W_S1_V = (signed long)voltEC[0] ;
      W_S2_V = (signed long)voltEC[1] ;
      break ;
    case 4: // EC導電率
      MeasEC() ;
      W_S1_V = (signed long)(condEC[0] * 1000) ;
      W_S2_V = (signed long)(condEC[1] * 1000) ;
      break ;
    default: // リセット
      initializeSetpoints() ;
      preVolume[0] = 0.0 ; // 排肥量計算用前回排液量をリセット
      preVolume[1] = 0.0 ; // 排肥量計算用前回排液量をリセット
      U_ccmList[CCMID_DFert1].value = 0 ; // 排肥量CCMリセット
      U_ccmList[CCMID_DFert2].value = 0 ;
      W_Mode = 1 ;
      oneMeasVol() ; // 測定
      W_S1_V = (signed long)Cdist[0] ;
      W_S2_V = (signed long)Cdist[1] ;
  }

  if(W_timeSetup_Mode == 1) { // 時刻設定
    RTCread() ;
    v_time = W_time ;
    RTCset() ;
    W_timeSetup_Mode = 0 ;
  }

  switch(W_dischg_Mode) { // 強制排出操作
    case 0:
      digitalWrite(DisChg_p, LOW) ;
      break ;
    case 1:
      dischgEndTime = W_dischg_Time + W_dischg_min ; // 自動排出終了時刻のセット
      if((dischgEndTime % 100) >= 60) dischgEndTime += 40 ; // 分桁上がり
      if(dischgEndTime >= 2400) dischgEndTime -= 2400 ; // 時桁上がり
      break ;
    case 2:
      digitalWrite(DisChg_p, HIGH) ;
      preVolume[0] = 0.0 ; // 排肥量計算用前回排液量をリセット
      preVolume[1] = 0.0 ; // 排肥量計算用前回排液量をリセット
      U_ccmList[CCMID_DFert1].value = 0 ; // 排肥量CCMリセット
      U_ccmList[CCMID_DFert2].value = 0 ;
      break ;
  }

  // 時刻読み込み
  RTCread() ;
  W_time = v_time ;
}

//------------------------------------------------------
//計測用各種関数
//超音波センサのハンドリングや換算値計算
//------------------------------------------------------
//温度計測 TH_Pinのアナログ入力の温度を測定
float MeasTemp(int TH_Pin) {
  float mv, temp ;
  mv = (float)analogRead(TH_Pin) *  5 / 1024 ;
  if((mv > 4.8)||(mv < 0.9)) { // temp. zone  -19,21 C to +64.37 C
    temp = 15.0 ; //温度計測不能なので音速340 m/sにセット
  }
  else {
     temp = (1.3679 * mv - 29.229) * mv + 89.57 ; //温度換算
  }
  return temp ;  
}

// 距離測定 trigPinとechoPinの超音波測距センサで距離計測 tempで温度補償
float MeasDist(int trigPin, int echoPin, float temp) {
  float dist ; // mm
  double dura ; // μs
  //距離測定
  //初期設定
  digitalWrite(trigPin, LOW); 
  delayMicroseconds(2); 
  digitalWrite( trigPin, HIGH ); //:計測命令を出力
  delayMicroseconds( 10 ); //
  //超音波発射
  digitalWrite( trigPin, LOW );
  //反射時間計測
  dura = pulseIn( echoPin, HIGH, 200000); //センサからの入力
  if (dura < 0) dist = 0.0 ; //エラー時は0に
  else {
    //距離をcm単位で計算
    dura = dura / 2; //往復距離を半分にする
    dist = (float)dura * (331.5 + 0.6 * temp) * 100 / 100000 ;
  }
  return dist ;
}

//排液量計算 距離mm 液量cc
float MeasVol(float d0, float d1, float d2, float v1, float v2, float d) {
  float r1m, r2m, r0, rd, vol ;
  v1 = v1 * 1000 ;
  v2 = v2 * 1000 ;
  r1m = sqrt(v1 / 3.14159 / (d0 - d1)) ;
  r2m = sqrt((v2 - v1) / 3.14159 / (d1 - d2)) ;
  r0 = r2m - ((r2m -r1m) / ((d0 - d2) / 2) * ((d0 - d1) + (d1 -d2) / 2)) ;
  rd = r0 + (r2m - r1m) / ((d0 - d2) / 2) * (d0 - d) ;
  vol = 3.14159 * (d0 - d) * (r0 * r0 + r0 * rd + rd * rd) / 3000 ;
  if(vol < 0) vol = 0.0 ; // 値が不適切ならば、液量を0にする
  if(d == 0.0) vol = 0.0 ; // 距離計測値エラーなら、液量を0にする
  return vol ;
}

// EC測定　voltEC[] mV  condEC[] dS/m
// センサ電極の分極を防ぐため1秒以上の再呼び出し待ちが必要。
// 引数posFがtrueなら順電圧, falseなら逆電圧で測定して前回との平均値
void MeasEC() {
  // 作業用変数設定
  float cellK[2] ;
  cellK[0] = (float)W_CellK1 / 100.0 ; // 小数点以下2桁なので
  cellK[1] = (float)W_CellK2 / 100.0 ;

  // ECセンサの電圧測定
  for(i = 0; i < 2; i++) { // チャンネル繰り返し
    ECVoutON(i) ; // 計測するチャンネルの電極電圧印加
    ECVmeasPos(i) ; // 電極セットアップ空測定
    voltEC[i] = ECVmeasNeg(i) ; // この後加電方向を変換しながら4回測定
    voltEC[i] += ECVmeasPos(i) ;
    voltEC[i] += ECVmeasNeg(i) ;
    voltEC[i] += ECVmeasPos(i) ;
    ECVmeasNeg(i) ; // 排液荷電方向リセットの空測定
    voltEC[i] /= 4.0 ; // 平均値算出
    ECVoutOFF() ; // 電極電圧印加停止
  }
  // 補償用液温測定
  for(i = 0; i < 2; i++) temperature[i] = MeasTemp(TH_p[i]) ;
  // 電気伝導率計算
  // R = 3300 (3.3 kΩ) V = 5000 mV で計算
  for(i = 0; i < 2; i++) {
    if(voltEC[i] >= 5000.0) {
      condEC[i] = 0 ;
    }
    else {
      condEC[i] = (10.0 * cellK[i] * (constV  - voltEC[i])) /
                  (constR * voltEC[i]) ; // dS/mで求める
      condEC[i] *= 1 + 0.02 * (temperature[i] - 25.0) ; // 温度補正
    }
  }
}

//EC測定　副処理
void ECVoutOFF() { // 印加電圧を断
  int ch ;
  for(ch = 0; ch < 2; ch++) {
    pinMode(EC_Vout[ch][0], INPUT);
    pinMode(EC_Vout[ch][1], INPUT);
  }
}

void ECVoutON(int ch) { // 印加電圧を接
  pinMode(EC_Vout[ch][0], OUTPUT); //出力モード
  pinMode(EC_Vout[ch][1], OUTPUT);
  digitalWrite(EC_Vout[ch][0], LOW) ; // オフ
  digitalWrite(EC_Vout[ch][1], LOW) ;
}

float ECVmeasPos(int ch) { // chに正方向電圧印加計測
  float measV ;
  digitalWrite(EC_Vout[ch][0], HIGH) ; // 順電圧印加
  digitalWrite(EC_Vout[ch][1], LOW) ;
  delay(Tsetup) ; // 計測開始待ち
  measV = 0.0; // 5回電圧測定した平均値
  for(int mlop = 0; mlop < 5; mlop++) {
    measV += (float)analogRead(EC_Vin[ch][1]) / 1023.0 * 5000.0 ;
    measV -= (float)analogRead(EC_Vin[ch][0]) / 1023.0 * 5000.0 ;
  }
  return measV / 5.0 ;
}

float ECVmeasNeg(int ch) { // chに逆方向電圧印加計測
  float measV ;
  digitalWrite(EC_Vout[ch][0], LOW) ; // 逆電圧印加
  digitalWrite(EC_Vout[ch][1], HIGH) ;
  delay(Tsetup) ; // 計測開始待ち
  measV = 0.0; // 5回電圧測定した平均値
  for(int mlop = 0; mlop < 5; mlop++) {
    measV += (float)analogRead(EC_Vin[ch][0]) / 1023.0 * 5000.0 ;
    measV -= (float)analogRead(EC_Vin[ch][1]) / 1023.0 * 5000.0 ;
  }
  return measV / 5.0 ;
}

// 排肥量計算
// dS/m と　増加分の液量から、meを求める。CCMに書き込む。
void calcME() {
  // EC値のCCM設定
  U_ccmList[CCMID_DECVal1].value = (signed long)(condEC[0] * 1000.0) ;
  U_ccmList[CCMID_DECVal2].value = (signed long)(condEC[1] * 1000.0) ;
  // 排液温度のCCM設定
  U_ccmList[CCMID_DNtemp1].value = (signed long)(temperature[0] * 10.0) ;
  U_ccmList[CCMID_DNtemp2].value = (signed long)(temperature[1] * 10.0) ;
  // 排肥量計算　0.1 dS m-1の時、1価イオンモル濃度が約1 me L-1
  float fert[2], volN[2] ; //
  volN[0] = (float)U_ccmList[CCMID_DLiquid1].value ;
  volN[1] = (float)U_ccmList[CCMID_DLiquid2].value ;
  for(i = 0; i < 2; i++) {
    if(preVolume[i] < volN[i]) { // 排液量が増加した時だけ計算
      // 単位がcm3なのでLで1/1000、かつmeへの換算はdSの×10、
      // そしてCCMのDFertが小数1位までで×10、よって1/10
      fert[i] = (volN[i] - preVolume[i]) * condEC[i] / 10.0 ;
      preVolume[i] = volN[i] ; // 計算済分の液量をセット
    }
    else fert[i] = 0.0 ; // 排肥がないのでリセット
  }
  // CCM変数に排肥量を加える(設定)
  U_ccmList[CCMID_DFert1].value += (signed long)fert[0] ;
  U_ccmList[CCMID_DFert2].value += (signed long)fert[1] ;
}

//RTCから日時読み込み
void RTCread() {
  Wire.requestFrom(RTC_ADRS,8);
  Wire.read();
  secondS = Wire.read();
  minuteS = Wire.read();
  hourS = Wire.read();
  v_time = ((long)(hourS >> 4) * 10 + (long)(hourS & 0xF)) * 100 + ((long)(minuteS >> 4) * 10 + (long)(minuteS & 0xF))  ;
  secondS = (secondS & 0x0F) + (secondS / 16 * 10) ;
  minuteS = (minuteS & 0x0F) + (minuteS / 16 * 10) ;
  hourS = (hourS & 0x0F) + (hourS / 16 * 10) ;
  Wire.read();
  dayS = Wire.read();
  months = Wire.read();
  years = Wire.read();
  v_date = ((long)(years >> 4) * 10 + (long)(years & 0xF)) * 10000 + ((long)(months >> 4) * 10 + (long)(months & 0xF)) * 100 + (long)(dayS >> 4) * 10 + (long)(dayS & 0xF) ;
  dayS = (dayS & 0x0F) + (dayS / 16 * 10) ;
  months = (months & 0x0F) + (months / 16 * 10) ;
  years = (years & 0x0F) + (years / 16 * 10) ;
}

//RTC日時書き込み
void RTCset() {
  byte bbuf ;
  Wire.beginTransmission(RTC_ADRS);
  Wire.write(0x00);
  Wire.write(0x00);  //sec
  bbuf = (byte)(v_time % 10) + (byte)(v_time / 10 % 10 * 16) ; //1と10の位
  Wire.write(bbuf);  //min
  bbuf = (byte)(v_time / 100 % 10) + (byte)(v_time / 1000 % 10 * 16) ; //100と1000の位
  Wire.write(bbuf);  //hour
  Wire.write(0x00);  //week 0:日曜 １:月曜 ・・・ 6:土曜
  bbuf = (byte)(v_date % 10) + (byte)(v_date / 10 % 10 * 16) ; //1と10の位
  Wire.write(bbuf);  //day
  bbuf = (byte)(v_date / 100 % 10) + (byte)(v_date / 1000 % 10 * 16) ; //100と1000の位
  Wire.write(bbuf);  //month
  bbuf = (byte)(v_date / 10000 % 10) + (byte)(v_date / 100000 % 10 * 16) ; //10000と100000の位
  Wire.write(bbuf);  //year
  Wire.endTransmission();
  delay(1);
}

//LCDにデータを送るサブルーチン
void LCD_write(byte t_data) {
  Wire.beginTransmission(LCD_ADRS);
  Wire.write(0x40);
  Wire.write(t_data);
  Wire.endTransmission();
  delay(1);
}

// LCDに文字列を表示させる
void LCD_string(char t_data[]) {
  for(i = 0; i < 16; i++) {
    Wire.beginTransmission(LCD_ADRS);
    Wire.write(0x40) ;
    Wire.write(t_data[i]) ;
    Wire.endTransmission() ;
    delay(1) ;
  }
}

//LCDに命令を実行させるサブルーチン
void LCD_command(byte t_command)
{
  Wire.beginTransmission(LCD_ADRS);
  Wire.write(0x00);
  Wire.write(t_command);
  Wire.endTransmission();
  delay(10);
}

void DispDist() {
  // 距離表示
  //1行目LCD表示値セット
  for(i=0 ; i<16 ; i++) lcd_buf[i]=' ' ; // 表示バッファクリア
  dtostrf(distance[0],5,0,&lcd_buf[0]) ; // 距離
  lcd_buf[5]= 'm' ;
  lcd_buf[6]= 'm' ;
  lcd_buf[7]= ',' ;
  dtostrf(distance[1],5,0,&lcd_buf[8]) ; // 距離
  lcd_buf[13]= 'm' ;
  lcd_buf[14]= 'm' ; 
  lcd_buf[15]= ' ' ;  
  //LCDに測定値表示
  LCD_command(0x80|0x00);
  for(i=0; i<16; i++) {
    LCD_write(lcd_buf[i]) ;
  }
}

void DispCurDist() {
  // 2行目に瞬時距離表示
  lcd_buf[0]= 0x0f ; // ©表示
  dtostrf(Cdist[0],5,0,&lcd_buf[1]) ; // 距離
  lcd_buf[6]= 'm' ;
  lcd_buf[7]= 'm' ;
  lcd_buf[8]= ',' ;
  dtostrf(Cdist[1],5,0,&lcd_buf[9]) ; // 距離
  lcd_buf[14]= 'm' ;
  lcd_buf[15]= 'm' ; 
  //LCDに測定値表示
  LCD_command(0x80|0x40);
  for(i=0; i<16; i++) {
    LCD_write(lcd_buf[i]) ;
  }
}

void DispTemp() {
  // 気温表示
  //1行目LCD表示値セット
  for(i=0 ; i<16 ; i++) lcd_buf[i]=' ' ; // 表示バッファクリア
  dtostrf(temperature[0],5,1,&lcd_buf[0]) ; // 気温
  lcd_buf[5]= 0xf2 ; // ℃表示
  lcd_buf[6]= 'C' ;
  lcd_buf[7]= ',' ;
  dtostrf(temperature[1],5,1,&lcd_buf[8]) ; // 気温
  lcd_buf[13]= 0xf2 ; // ℃表示
  lcd_buf[14]= 'C' ;
  lcd_buf[15]= ' ' ;
  //LCDに測定値表示
  LCD_command(0x80|0x00);
  for(i=0; i<16; i++) {
    LCD_write(lcd_buf[i]) ;
  }
}

void DispVol() {
  //排液量表示
  for(i=0 ; i<16 ; i++) lcd_buf[i]=' ' ; // 表示バッファクリア
  dtostrf(volume[0],5,0,&lcd_buf[0]) ; // 液量
  lcd_buf[5]= 'c' ;
  lcd_buf[6]= 'c' ;
  lcd_buf[7]= ',' ;
  dtostrf(volume[1],5,0,&lcd_buf[8]) ; // 液量
  lcd_buf[13]= 'c' ;
  lcd_buf[14]= 'c' ; 
  lcd_buf[15]= ' ' ;  
  //LCDに測定値表示
  LCD_command(0x80|0x00);
  for(i=0; i<16; i++) {
    LCD_write(lcd_buf[i]) ;
  }
}

void DispCurVol() {
  //排液量瞬時値表示
  lcd_buf[0]= 0x0f ; // ©表示
  dtostrf(Cvol[0],5,0,&lcd_buf[1]) ; // 距離
  lcd_buf[6]= 'c' ;
  lcd_buf[7]= 'c' ;
  lcd_buf[8]= ',' ;
  dtostrf(Cvol[1],5,0,&lcd_buf[9]) ; // 距離
  lcd_buf[14]= 'c' ;
  lcd_buf[15]= 'c' ; 
  //LCD2行目に測定値表示
  LCD_command(0x80|0x40);
  for(i=0; i<16; i++) {
    LCD_write(lcd_buf[i]) ;
  }
}

void DispECvolt() {
  float cellk ;
  //EC電圧とcellコンスタント表示
  for(i=0 ; i<16 ; i++) lcd_buf[i]=' ' ; // 表示バッファクリア
  //1行目に電圧
  lcd_buf[0]= ' ' ;
  dtostrf(voltEC[0],5,0,&lcd_buf[1]) ; // 電圧1
  lcd_buf[6]= 'm' ;
  lcd_buf[7]= 'V' ;
  lcd_buf[8]= ',' ;
  dtostrf(voltEC[1],5,0,&lcd_buf[9]) ; // 電圧2
  lcd_buf[14]= 'm' ;
  lcd_buf[15]= 'V' ; 
  //LCD1行目に測定値表示
  LCD_command(0x80|0x00);
  for(i=0; i<16; i++) {
    LCD_write(lcd_buf[i]) ;
  }
  //2行目にcellコンスタント
  lcd_buf[0]= 'c' ;
  lcd_buf[1]= 'e' ;
  lcd_buf[2]= 'l' ;
  lcd_buf[3]= 'l' ;
  lcd_buf[4]= 'K' ;
  cellk = (float)W_CellK1 / 100.0 ;
  dtostrf(cellk,5,2,&lcd_buf[5]) ; // 係数1
  lcd_buf[10]= ',' ;
  cellk = (float)W_CellK2 / 100.0 ;
  dtostrf(cellk,5,2,&lcd_buf[11]) ; // 係数2
  //LCD2行目に測定値表示
  LCD_command(0x80|0x40);
  for(i=0; i<16; i++) {
    LCD_write(lcd_buf[i]) ;
  }
}

void DispECcond() {
  // EC導電率表示
  //1行目に排肥量表示
  float me1, me2 ;
  me1 = (float)U_ccmList[CCMID_DFert1].value / 10.0 ;
  me2 = (float)U_ccmList[CCMID_DFert2].value / 10.0 ;  
  for(i=0 ; i<16 ; i++) lcd_buf[i]=' ' ; // 表示バッファクリア
  //1行目に電圧
  lcd_buf[0]= ' ' ;
  dtostrf(me1,5,1,&lcd_buf[1]) ; // 排肥量1
  lcd_buf[6]= 'm' ;
  lcd_buf[7]= 'e' ;
  lcd_buf[8]= ',' ;
  dtostrf(me2,5,1,&lcd_buf[9]) ; // 排肥量2
  lcd_buf[14]= 'm' ;
  lcd_buf[15]= 'e' ; 
  //LCD1行目に測定値表示
  LCD_command(0x80|0x00);
  for(i=0; i<16; i++) {
    LCD_write(lcd_buf[i]) ;
  }
  //2行目EC導電率表示値セット
  dtostrf(condEC[0],5,2,&lcd_buf[0]) ; // EC1
  lcd_buf[5]= 'd' ;
  lcd_buf[6]= 'S' ;
  lcd_buf[7]= ',' ;
  dtostrf(condEC[1],5,2,&lcd_buf[8]) ; // EC2
  lcd_buf[13]= 'd' ;
  lcd_buf[14]= 'S' ; 
  lcd_buf[15]= ' ' ;  
  //LCD2行目に測定値表示
  LCD_command(0x80|0x40);
  for(i=0; i<16; i++) {
    LCD_write(lcd_buf[i]) ;
  }
}

void DispTime() {
  //時間関係表示
  dtostrf(hourS,2,0,&lcd_buf[0]) ; // 現在時刻
  if(lcd_buf[0] == ' ') lcd_buf[0] = '0' ;
  lcd_buf[2]= ':' ;
  dtostrf(minuteS,2,0,&lcd_buf[3]) ;
  if(lcd_buf[3] == ' ') lcd_buf[3] = '0' ;
  lcd_buf[5]= ':' ;
  dtostrf(secondS,2,0,&lcd_buf[6]) ;
  if(lcd_buf[6] == ' ') lcd_buf[6] = '0' ;
  lcd_buf[8]= '(' ;
  lcd_buf[9]= 0x7e ;
  dtostrf(W_dischg_Time / 100,2,0,&lcd_buf[10]) ; // 排出時刻
  if(lcd_buf[10] == ' ') lcd_buf[10] = '0' ;
  lcd_buf[12]= ':' ;
  dtostrf(W_dischg_Time % 100,2,0,&lcd_buf[13]) ;
  if(lcd_buf[13] == ' ') lcd_buf[13] = '0' ;
  lcd_buf[15]= ')' ; 
  //LCD2行目に測定値表示
  LCD_command(0x80|0x40);
  for(i=0; i<16; i++) {
    LCD_write(lcd_buf[i]) ;
  }
}

//
// 2セットの1回分の温度距離測定
//
void OneShotTD() {
  //距離測定
  for(ch = 0; ch < 2; ch++) {
    temperature[ch] = MeasTemp(TH_p[ch]) ;
    Cdist[ch] = MeasDist(t_p[ch], e_p[ch], temperature[ch]) ;
  }
}

//
// 2セットの排液量計算
//
void calcVol() {
  volume[0] = MeasVol((float)W_S1_EmpDis, (float)W_S1Adj1_Dis, (float)W_S1Adj2_Dis,(float)W_Adj1_Vol, (float)W_Adj2_Vol, distance[0]) ;
  volume[1] = MeasVol((float)W_S2_EmpDis, (float)W_S2Adj1_Dis, (float)W_S2Adj2_Dis,(float)W_Adj1_Vol, (float)W_Adj2_Vol, distance[1]) ;
}
// 瞬時値用
void CcalcVol() {
  Cvol[0] = MeasVol((float)W_S1_EmpDis, (float)W_S1Adj1_Dis, (float)W_S1Adj2_Dis,(float)W_Adj1_Vol, (float)W_Adj2_Vol, Cdist[0]) ;
  Cvol[1] = MeasVol((float)W_S2_EmpDis, (float)W_S2Adj1_Dis, (float)W_S2Adj2_Dis,(float)W_Adj1_Vol, (float)W_Adj2_Vol, Cdist[1]) ;
}

//
// 押しボタンの計測で瞬時値を計測する
//
void oneMeasVol() {
  OneShotTD() ;
  CcalcVol() ;
}

//
//  距離を1回測定し、測定値を配列にソートする
//
void oneShotMeas() {
  float swapv, workv ; // 入れ替え作業変数
  OneShotTD() ;
  if(nofMD == 0) {
    //初回測定
    for(i = 0; i < 2; i++) rowDist[i][0] = Cdist[i] ;
  }
  else {
    // 2回目以降
    for(i = 0; i < 2; i++) {
      workv = Cdist[i] ; // 作業変数に代入
      for(j = 0; j < nofMD; j++) {
        // 値が小さい順に並べる
        if(rowDist[i][j] > workv) {
          // 入れ替え
          swapv = rowDist[i][j] ;
          rowDist[i][j] = workv ;
          workv = swapv ;
        }
      }
      rowDist[i][nofMD] = workv ;
    }
  }
  nofMD++ ; // 測定回数を1増やす    
}

//
// これまで計測した距離のメジアン値から排液量を計算し、UECS-CCM変数に代入
//
void measurementVol() {
  if(nofMD == 0) oneShotMeas() ; // 1個も測定値がなければ測定する。
  j = (nofMD - 1)/ 2 ; // 中央値の配列のインデックスを計算
  for(i = 0; i < 2; i++) distance[i] = rowDist[i][j] ; // 中央値を代入
  if(j >= 5) { // 10個以上測定値があれば、中央値1個と前後1個の計3個の平均値にする。
    for(i = 0; i < 2; i++) {
      for(k = 1; k < 2; k++) {
        distance[i] += rowDist[i][j-k] ;
        distance[i] += rowDist[i][j+k] ;
      }
      distance[i] /= 3 ;
    }
  }    
  // 距離のメジアン値を容量換算し、CCMに代入
  calcVol() ;
  // 前回より低い排液量が計測されたときは前回の排液量のままとする(v.1.5)
  if(preVolume[0] > volume[0]) volume[0] = preVolume[0] ;
  if(preVolume[1] > volume[1]) volume[1] = preVolume[1] ;
  U_ccmList[CCMID_DLiquid1].value = (signed long)volume[0] ;
  U_ccmList[CCMID_DLiquid2].value = (signed long)volume[1] ;
  // 計測回数をリセット
  nofMD = 0 ;
}  


//---------------------------------------------------------
//setup()実行後に呼び出されるメインループ
//この関数内ではUECSloop()関数を呼び出さなくてはならない。
//UserEveryLoop()に似ているがネットワーク関係の処理を行う前に呼び出される。
//必要に応じて処理を記述してもかまわない。
//呼び出される頻度が高いため,重い処理を記述しないこと。
//---------------------------------------------------------
void loop(){
  
  UECSloop();
  
}

//---------------------------------------------------------
//液晶表示
//---------------------------------------------------------
void ShowLCD() {
  switch(dispMode) { // 液晶表示
    case 0: DispTemp() ; // 気温表示
            DispTime() ; // 時刻関連表示
            break ;
    case 1: DispDist() ; // メジアン処理した距離表示
            DispCurDist() ; // 瞬時値の距離表示
            break ;
    case 2: DispVol() ; // メジアン処理した排液量表示
            CcalcVol() ; // 距離瞬時値の容量換算
            DispCurVol() ;  // 瞬時値の排液量表示
            break ;
    case 3: DispECvolt() ; // EC電圧値表示
            break ;
    case 4: DispECcond() ; // EC導電率表示
            break ;
  }
}

//---------------------------------------------------------
//メインループ
//システムのタイマカウント，httpサーバーの処理，
//UDP16520番ポートと16529番ポートの通信文をチェックした後，呼び出さされる関数。
//呼び出される頻度が高いため，重い処理を記述しないこと。
//---------------------------------------------------------
void UserEveryLoop(){
  
  if(swFlag == 0) { //押しボタンで表示切替
    if(digitalRead(PB_p) == 0) {
      swFlag = 1;
      if(++dispMode >= 5) dispMode = 0 ;
      ShowLCD() ;
    }
  }
  else {
    if(digitalRead(PB_p) == 1) {
      swFlag = 0 ;
    }
  }

}

//---------------------------------------------------------
//１分に１回呼び出される関数
//---------------------------------------------------------
void UserEveryMinute(){
  
  measurementVol() ; // 測定距離のメジアン値を排液量に換算してCCMに格納
  MeasEC() ; // EC測定
  calcME() ; // 排肥量計算とECとともにCCMに格納
}

//---------------------------------------------------------
//１秒に１回呼び出される関数
//---------------------------------------------------------
void UserEverySecond(){

  RTCread() ; // 毎秒の時刻更新
  W_time = v_time ; 

  oneShotMeas() ; // 1回の距離測定
    
  ShowLCD() ; // 液晶表示

  U_ccmList[CCMID_cnd].value = 0 ; // cnd CCMの値設定

  if(W_dischg_Mode == 1) { // 自動排出モードの処理
    if(W_dischg_Time <= dischgEndTime) { // 日をまたがない
      if((v_time >= W_dischg_Time)&&(v_time < dischgEndTime)) {
       digitalWrite(DisChg_p, HIGH) ;
        preVolume[0] = 0.0 ; // 排肥量計算用前回排液量をリセット
        preVolume[1] = 0.0 ; // 排肥量計算用前回排液量をリセット
        U_ccmList[CCMID_DFert1].value = 0 ; // 排肥量CCMリセット
        U_ccmList[CCMID_DFert2].value = 0 ;
      }
      else digitalWrite(DisChg_p, LOW) ;
    }
    else { //　日をまたぐ
      if((v_time < dischgEndTime)||(v_time >= W_dischg_Time)) { // v17修正
        digitalWrite(DisChg_p, HIGH) ;
        preVolume[0] = 0.0 ; // 排肥量計算用前回排液量をリセット
        preVolume[1] = 0.0 ; // 排肥量計算用前回排液量をリセット
        U_ccmList[CCMID_DFert1].value = 0 ; // 排肥量CCMリセット
        U_ccmList[CCMID_DFert2].value = 0 ;     
      }
      else digitalWrite(DisChg_p, LOW) ;
    }
  }
}
