//---------------------------------------------------//
//  Low-cost UECS nutrient supplier  node
//                                       (LU-NTSUP##)
//
// UARDECS_MEGAを使用して、
// Arduino mega 2560 + Arduino Ethernet 2 専用です
//
// Version 1.0 2017.09.15 Progammed by T.Hoshi
// version 1.0a 2017.10.31
//   強制給液時間間隔を設定していた時、日付が変わって
//   前回給液時刻が0時にリセットされてしまうので、夜明け時に強制
//   給液してしまった。それを防止するために、対策を行った。
// Version 1.0b 2018.8.10
//   太陽電池のシャント抵抗を3.3Ωにしたため、日射換算値を変更
// Version 1.1 2019.1.16  3点の改良を実施
//   ・1回の給液時間を秒単位で設定可能にする。
//   ・給液定数の設定単位をL/株/分からcc/株/分にする。
//   ・時刻を外部参照できるようにする。内蔵クロック不要に。
// Version 2.0 (VC) 2019.9.15
//   給液時刻を5個設置
//   自動と時刻の給肥濃度をそれぞれ％単位で設定可能
// Version 2.1 (VC1)　2020.7.22
//   日射比例給液の設定を0から20000迄延長
// Version 3.0 (VC2) 2021.8.20
//   肥料原液(100%濃度)と原水(0%濃度)のECを設定し
//   meの吸肥量　TdMEIntcNM  cm3給液量 TdLQInt.cNM を設置
// Version 3.1 CCMのme/plantの小数点以下を4桁にした。日給液
//   量の小数点以下1桁増やした。
// Version 3.2 ECからme/Lの換算係数を設定可能に
//---------------------------------------------------//
#define verNO   "LU-NTSUP32" // 10 char.
#define verDATE "20230921" // 8 char.
// このプログラムは、無償提供する代わりに、動作結果に関する
// 一切の保証は致しません。自己責任でお使いください。また、
// 改造も自由ですが、公刊等をされる場合は、「近畿大学　星　
// 岳彦が開発したLU-NTSUP##プログラムを用いた」と引用して下さい
// ますと、今後の研究開発の励みになりますので、どうぞよろ
// しくお願いします。
//
/*
-----------------------------------------------------------------------------
LU-NTSUP## software are distributed under the 2 clause BSD license.

Copyright 2023, Takehiko Hoshi All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.

-----------------------------------------------------------------------------
LU-NTSUP##ソフトウェアは、2条項BSDライセンスの下に配布されます。

Copyright 2023, Takehiko Hoshi All rights reserved.

  ソースコード形式かバイナリ形式か、変更するかしないかを問わず、以下の条件を
満たす場合に限り、再頒布および使用が許可されます。

1.ソースコードを再頒布する場合、上記の著作権表示、本条件一覧、および下記免責
  条項を含めること。
2.バイナリ形式で再頒布する場合、頒布物に付属のドキュメント等の資料に、上記の
  著作権表示、本条件一覧、および下記免責条項を含めること。

  本ソフトウェアは、著作権者およびコントリビューターによって「現状のまま」提供
されており、明示黙示を問わず、商業的な使用可能性、および特定の目的に対する適合
性に関する暗黙の保証も含め、またそれに限定されない、いかなる保証もありません。
著作権者もコントリビューターも、事由のいかんを問わず、 損害発生の原因いかんを
問わず、かつ責任の根拠が契約であるか厳格責任であるか（過失その他の）不法行為で
あるかを問わず、仮にそのような損害が発生する可能性を知らされていたとしても、
本ソフトウェアの使用によって発生した （代替品または代用サービスの調達、使用の
喪失、データの喪失、利益の喪失、業務の中断も含め、またそれに限定されない）直接
損害、間接損害、偶発的な損害、特別損害、懲罰的損害、または結果損害について、
一切責任を負わないものとします。
-----------------------------------------------------------------------------
*/
// このシールドはArduino MEGA 2560 + Ethernet Shiedld2の
// 組み合わせで動作させることを前提にしています。
// Ethernet Sheild 2基板のシールのmacアドレスに変更して
// ください。macアドレスとは。90:A2:DA:0F:xx:xx のような8個
// の16進数です。これをばらして下の0xの後に転記してください。
#define mac0 0xA8
#define mac1 0x61
#define mac2 0x0A
#define mac3 0xAE
#define mac4 0x15
#define mac5 0x54

// UECS用のライブラリUARDECSのインストール
// https://uecs.org/arduino/uardecs.html が必要です。
#include <Uardecs_mega.h>
#include <SPI.h>
#include <Ethernet2.h>
#include <avr/pgmspace.h>
#include <EEPROM.h>
#include <Wire.h>

//////////////////////
// 定数関連定義
//////////////////////

// 計測信号線の定義
//センサ・SD信号線定義
//analog
const uint8_t srsPin  =  0 ; //太陽電池 光関係
// I2Cデバイスアドレスの定義
#define RTC_ADRS  0x32 //リアルタイムクロック

// UARDECS用IP初期化　
// 低コストUECS制御シールド用設定 セーフモードピンを
// ショートピン(A3)を抜くとセーフモード(192.168.1.7)
// analog A3 (=D57) pin for Safe mode
const byte U_InitPin = 57 ; // for Arduino MEGA 2560 
const byte U_InitPin_Sense=HIGH;

// UARDECSのUECSノード情報設定
const char U_name[] PROGMEM = verNO;
const char U_vender[] PROGMEM = "Hoshi-lab.";
const char U_uecsid[] PROGMEM = "01100900100D";
const char U_footnote[] PROGMEM = "Product with UARDECS200(MEGA) by <A HREF=\"http://hoshi-lab.info/\">Kindai, hoshi-lab.</A>";
char U_nodename[20] = verNO;
UECSOriginalAttribute U_orgAttribute;

////////////////////////////////////////////
// 培養液供給ノードの変数設定とライブラリ
////////////////////////////////////////////

// 入出力ポートの定義
const int port[4] = { 7, 8, 5, 6 } ;
// ウオッチドッグタイマーのリセットポート
const int wdt = 9 ; // 0 = clear timer
// ディジタル出力値の一時記憶装置のトリガポート
const int ratch = 3 ; //  0->1 ratch
// ディジタル入力値のイネーブルポート
const int in_out = 2 ; // in = 1 out =0

// 作業変数
int i ; // ループ用変数
float mv ; // AD変換用作業変数
signed long intsupply ; // 1分間の秒ごとの給肥動作時の給肥定数積算値
                        // 総給肥量の計算に必要
signed long intsupplyw ; // 1分間の秒ごとの給水動作時の給肥定数積算値
                        // 総給水量の計算に必要

// 自動給肥時間の残(秒)
signed long rNutSec ; //毎秒これが0以上なら給肥する
signed long rWtSec ;  //毎秒これが0以上なら給水する 給肥優先

//Define variables for Web interface
signed long W_supply ; // 供給状態 0:停止 1:液肥供給 2:水供給
signed long W_mode ; // 給肥モード
signed long W_modeset ; // 給液モード設定
signed long W_ntconst ; // 給液定数 ml/株/分 0.1 ml単位(1/10)
signed long W_onesp ; // 給液時間 単位:秒
signed long W_nf_ratio ; // 時刻培養液濃度(%)
signed long W_na_ratio ; // 自動培養液濃度(%)
signed long W_ntspsrint ; // 給肥積算日射
signed long W_ntfcspint ; // 補填給液間隔
signed long W_fctime1 ; // 給肥時刻1
signed long W_fctime2 ; // 給肥時刻2
signed long W_fctime3 ; // 給肥時刻3
signed long W_fctime4 ; // 給肥時刻4
signed long W_fctime5 ; // 給肥時刻5
signed long W_time_in ; // 内部時刻
signed long W_setdtf ; // 日時設定・時計モード0:外部、1:内部、2:内部設定
signed long W_useoutsrf ; // 外部日射使用
signed long W_sr_in ; // 内部日射量
signed long W_pnrtime ; // 前回給肥水時刻
signed long W_srint_pr ; // 現在積算日射
signed long W_srint_td ; // 本日積算日射
signed long W_amnt_td ; // 本日給肥量
signed long W_amnt_yd ; // 昨日給肥量
signed long W_amnt_all ; // 総給肥量
signed long W_amw_td ; // 本日給水量
signed long W_amw_yd ; // 昨日給水量
signed long W_amw_all ; // 総給水量
signed long W_waterEC; //  原水EC
signed long W_nutriEC; // 培養液EC
signed long W_EC2meL ; // ECからme/Lへの変換係数

// ウオッチドッグタイマーのクリア(2秒以内に1回の実行が必要)
void clearWdt() {
  digitalWrite(wdt, LOW) ;
  delayMicroseconds(100) ;
  digitalWrite(wdt, HIGH) ;
}

// ディジタル値を出力する d[0]-d[3]の値 0はOFF、1はON
void dout(int *d) {
  digitalWrite(in_out, LOW) ;
  for(i = 0; i < 4; i++) {
    pinMode(port[i], OUTPUT) ;
    digitalWrite(port[i], d[i]) ;
  }
  delayMicroseconds(1) ;
  digitalWrite(ratch, HIGH) ;
  delayMicroseconds(1) ; 
  digitalWrite(ratch, LOW) ;
  for(i = 0; i < 4; i++) {
    pinMode(port[i], INPUT) ;
  }
  digitalWrite(in_out, HIGH) ;
}

// ディジタル出力を全てOFFにする
void clearDout() {
  int d[4] = { 
    LOW, LOW, LOW, LOW
  } ;
  dout(d) ;
}

// 出力の割り付け状態 低コストUECS制御シールド出力端子
// 変数　リレー　端子台 PNo.
// d[0] RL1   CN2   3-4 培養液供給信号　ONの時だけ給肥
// d[1] RL2   CN2   1-2 給水信号　ONの時だけ給水
// d[2] RL3   CN1   3-4 
// d[3] RL4   CN1   1-2 
 
// 養液供給
void nutrientON() {
  int d[4] = { 
    HIGH, LOW, LOW, LOW
  } ;
  dout(d) ;
  W_supply = 1 ;
}
  
// 養液停止・水停止
void nutrientOFF() { // waterOFF()も兼ねる
  clearDout() ;
  W_supply = 0 ;
}

// 水供給
void waterON() {
  int d[4] = { 
    LOW, HIGH, LOW, LOW
  } ;
  dout(d) ;
  W_supply = 2 ;
}

// ディジタル入力値の読み込み d[0]-d[3]に値を代入
void din(int *d) {
  digitalWrite(in_out, HIGH) ;
  delayMicroseconds(1) ;
  for(i = 0; i < 4; i++) {
    pinMode(port[i], INPUT) ;
    delayMicroseconds(1) ;
    d[i] = digitalRead(port[i]) ;
  }
}

/////////
// RTC
/////////

//RTC関係グローバル変数
signed long v_date ; //日付
signed long v_time ; //時刻
byte seconds,minutes,pminutes,hours,days,months,years;
// 秒カウンター
int Csec ; // 1分を計時するため

//RTCから日時読み込み
void RTCread() {
  Wire.requestFrom(RTC_ADRS,8);
  Wire.read();
  seconds = Wire.read();
  minutes = Wire.read();
  hours = Wire.read();
  v_time = ((long)(hours >> 4) * 10 + (long)(hours & 0xF)) * 100 + ((long)(minutes >> 4) * 10 + (long)(minutes & 0xF))  ;
  Wire.read();
  days = Wire.read();
  months = Wire.read();
  years = Wire.read();
  v_date = ((long)(years >> 4) * 10 + (long)(years & 0xF)) * 10000 + ((long)(months >> 4) * 10 + (long)(months & 0xF)) * 100 + (long)(days >> 4) * 10 + (long)(days & 0xF) ;
}

//RTC日時書き込み
void RTCset() {
  byte bbuf ;
  Wire.beginTransmission(RTC_ADRS);
  Wire.write(0x00);
  Wire.write(0x00);  //sec
  bbuf = (byte)(v_time % 10) + (byte)(v_time / 10 % 10 * 16) ; //1と10の位
  Wire.write(bbuf);  //min
  bbuf = (byte)(v_time / 100 % 10) + (byte)(v_time / 1000 % 10 * 16) ; //100と1000の位
  Wire.write(bbuf);  //hour
  Wire.write(0x00);  //week 0:日曜 １:月曜 ・・・ 6:土曜
  bbuf = (byte)(v_date % 10) + (byte)(v_date / 10 % 10 * 16) ; //1と10の位
  Wire.write(bbuf);  //day
  bbuf = (byte)(v_date / 100 % 10) + (byte)(v_date / 1000 % 10 * 16) ; //100と1000の位
  Wire.write(bbuf);  //month
  bbuf = (byte)(v_date / 10000 % 10) + (byte)(v_date / 100000 % 10 * 16) ; //10000と100000の位
  Wire.write(bbuf);  //year
  Wire.endTransmission();
  delay(1);
}

//time1 - time2 の経過分数
signed long elapsMin(signed long time1, signed long time2) {
  signed long wk ;
  wk = ((time1 / 100 * 60) + (time1 % 100)) - ((time2 / 100 * 60) + (time2 % 100)) ;
  if(wk < 0) wk += 1440 ;
  return wk ;
}

//////////////////////////////////
// 計測制御基本ライブラリ
//////////////////////////////////

// 太陽電池で日射量測定 
signed long SRmeasure() {
  mv = (float)analogRead(srsPin) *  5000 / 1023 ;
  if(mv < 0) mv = 0.0 ;
   // solar radiation convert
   // for 3.3Ω 2018.8.10
  mv = 0.6704 * mv * 2 ; // W m-2
  return  (signed long)mv ; 
}

//////////////////////////////////
// html page setting
//////////////////////////////////

// Define Strings
const char S_supply[] PROGMEM =  "供給状態";
const char S_modedp[] PROGMEM =  "動作モード";
const char S_modeset[] PROGMEM =  "動作設定";
const char S_const[] PROGMEM =  "給液定数";
const char S_time[] PROGMEM =  "給液時間";
const char S_spintsr[] PROGMEM =  "給液積算日射";
const char S_intfcntsup[] PROGMEM =  "補填給液間隔";
const char S_fcntsup1[] PROGMEM =  "給液時刻1";
const char S_fcntsup2[] PROGMEM =  "給液時刻2";
const char S_fcntsup3[] PROGMEM =  "給液時刻3";
const char S_fcntsup4[] PROGMEM =  "給液時刻4";
const char S_fcntsup5[] PROGMEM =  "給液時刻5";
const char S_insidedt[] PROGMEM =  "日付";
const char S_insidetm[] PROGMEM =  "時刻";
const char S_setdttm[] PROGMEM =  "日時";
const char S_useoutsr[] PROGMEM =  "日射使用";
const char S_insidesr[] PROGMEM =  "日射量";
const char S_prenttime[] PROGMEM =  "直近給液";
const char S_intsrnow[] PROGMEM =  "現在積算日射";
const char S_intsrtd[] PROGMEM =  "本日積算日射";
const char S_ntsuptd[] PROGMEM =  "本日給肥量";
const char S_ntsupyd[] PROGMEM =  "昨日給肥量";
const char S_amntsup[] PROGMEM =  "総給肥量";
const char S_wsuptd[] PROGMEM =  "本日給水量";
const char S_wsupyd[] PROGMEM =  "昨日給水量";
const char S_amwsup[] PROGMEM =  "総給水量";
const char S_na_ratio[] PROGMEM =  "自動液肥濃度";
const char S_nf_ratio[] PROGMEM =  "時刻液肥濃度";
const char S_waterEC[] PROGMEM = "原水EC";
const char S_nutriEC[] PROGMEM = "培養液EC";
const char S_EC2meL[] PROGMEM = "1dS/mは" ;

//Select Strings
const char S_off[] PROGMEM = "停止";
const char S_on[] PROGMEM = "液肥供給";
const char S_onW[] PROGMEM = "水供給";
const char S_auto[] PROGMEM = "自動";
const char S_stop[] PROGMEM = "強制停止";
const char S_force[] PROGMEM = "強制給肥";
const char S_Wforce[] PROGMEM = "強制給水";
const char S_reset[] PROGMEM = "リセット";
const char S_remoteC[] PROGMEM = "CPU遠隔";
const char S_remoteS[] PROGMEM = "手動SW";
const char S_web[] PROGMEM = "下記動作";
const char S_ins[] PROGMEM = "内部設定";
const char S_in[] PROGMEM = "内部";
const char S_out[] PROGMEM = "外部";
const char S_outs[] PROGMEM = "外部設定";
const char *SS_SUP[3]={ S_off, S_on, S_onW };
const char *SS_MDDP[4]={ S_auto, S_remoteC, S_remoteS, S_web };
const char *SS_MDSET[5]={ S_auto, S_stop, S_force, S_Wforce, S_reset };
const char *SS_ISO[4]={ S_in, S_ins, S_out, S_outs };
const char *SS_IO[2]={ S_in, S_out };

//Unit Strings
const char U_perlm[] PROGMEM =  "mℓ/株/分";
const char U_perl[] PROGMEM =  "ℓ/株";
const char U_seci[] PROGMEM =  "秒間";
const char U_minaf[] PROGMEM =  "分後";
const char U_eng[] PROGMEM =  "kJ/m2";
const char U_dpereng[] PROGMEM =  "kJ/m2/日";
const char U_flux[] PROGMEM =  "kW/m2";
const char U_time[] PROGMEM =  "時.分";
const char U_sec[] PROGMEM =  "秒";
const char U_percent[] PROGMEM =  "%";
const char U_EC[] PROGMEM = "dS/m";
const char U_EC2meL[] PROGMEM = "me/L相当" ;

//Detail Strings
const char D_1time[] PROGMEM =   "1回で";
const char D_sup1time[] PROGMEM =   "毎に給液";
const char D_fb2400[] PROGMEM =   "24.00で禁止";
const char D_zero[] PROGMEM =   "0は禁止";
const char D_prev[] PROGMEM =   "終了時刻";
const char D_order[] PROGMEM =   "水→肥";

// Dummy Strings
const char EMPTY[] PROGMEM= "";
const char** DUMMY = NULL;

const int U_HtmlLine = 30; //Total number of HTML table rows.
struct UECSUserHtml U_html[U_HtmlLine]={
//{名前,          入出力形式,     単位,       詳細説明,     選択肢文字列,選択肢数, 値,             最小値,最大値, 小数桁数}
  {S_supply,      UECSSHOWSTRING, EMPTY,      EMPTY,        SS_SUP,      3,        &(W_supply),    0,     1,             0},
  {S_modedp,      UECSSHOWSTRING, EMPTY,      EMPTY,        SS_MDDP,     4,        &(W_mode),      0,     3,             0},
  {S_modeset,     UECSSELECTDATA, EMPTY,      EMPTY,        SS_MDSET,    5,        &(W_modeset),   0,     3,             0},
  {S_time,        UECSINPUTDATA,  U_seci,     D_1time,      DUMMY,       0,        &(W_onesp),     1,     1200,          0},
  {S_insidesr,    UECSSHOWDATA,   U_flux,     EMPTY,        DUMMY,       0,        &(W_sr_in),     0,     0,             3},
  {S_useoutsr,    UECSSELECTDATA, EMPTY,      EMPTY,        SS_IO,       2,        &(W_useoutsrf), 0,     1,             0},
  {S_intsrnow,    UECSSHOWDATA,   U_eng,      EMPTY,        DUMMY,       0,        &(W_srint_pr),  0,     0,             3},
  {S_spintsr,     UECSINPUTDATA,  U_eng,      D_sup1time,   DUMMY,       0,        &(W_ntspsrint), 1,     20000,         0},
  {S_intfcntsup,  UECSINPUTDATA,  U_minaf,    D_zero,       DUMMY,       0,        &(W_ntfcspint), 0,     720,           0},
  {S_na_ratio,     UECSINPUTDATA, U_percent,  D_order,      DUMMY,       0,        &(W_na_ratio),  0,     100,           0},
  {S_intsrtd,     UECSSHOWDATA,   U_eng,      EMPTY,        DUMMY,       0,        &(W_srint_td),  0,     0,             3},
  {S_insidetm,    UECSINPUTDATA,  U_time,     EMPTY,        DUMMY,       0,        &(W_time_in),   0,     2359,          2},
  {S_setdttm,     UECSSELECTDATA, EMPTY,      EMPTY,        SS_ISO,      4,        &(W_setdtf),    0,     3,             0},
  {S_fcntsup1,    UECSINPUTDATA,  U_time,     D_fb2400,     DUMMY,       0,        &(W_fctime1),   0,     2400,          2},
  {S_fcntsup2,    UECSINPUTDATA,  U_time,     D_fb2400,     DUMMY,       0,        &(W_fctime2),   0,     2400,          2},
  {S_fcntsup3,    UECSINPUTDATA,  U_time,     D_fb2400,     DUMMY,       0,        &(W_fctime3),   0,     2400,          2},
  {S_fcntsup4,    UECSINPUTDATA,  U_time,     D_fb2400,     DUMMY,       0,        &(W_fctime4),   0,     2400,          2},
  {S_fcntsup5,    UECSINPUTDATA,  U_time,     D_fb2400,     DUMMY,       0,        &(W_fctime5),   0,     2400,          2},
  {S_nf_ratio,    UECSINPUTDATA,  U_percent,  D_order,      DUMMY,       0,        &(W_nf_ratio),  0,     100,           0},
  {S_prenttime,   UECSSHOWDATA,   U_time,     D_prev,       DUMMY,       0,        &(W_pnrtime),   0,     0,             2},
  {S_const,       UECSINPUTDATA,  U_perlm,    EMPTY,        DUMMY,       0,        &(W_ntconst),   0,     1000,          1},  
  {S_ntsuptd,     UECSSHOWDATA,   U_perl,     EMPTY,        DUMMY,       0,        &(W_amnt_td),   0,     0,             4},
  {S_ntsupyd,     UECSSHOWDATA,   U_perl,     EMPTY,        DUMMY,       0,        &(W_amnt_yd),   0,     0,             4},
  {S_amntsup,     UECSSHOWDATA,   U_perl,     EMPTY,        DUMMY,       0,        &(W_amnt_all),  0,     0,             4},
  {S_wsuptd,      UECSSHOWDATA,   U_perl,     EMPTY,        DUMMY,       0,        &(W_amw_td),    0,     0,             4},
  {S_wsupyd,      UECSSHOWDATA,   U_perl,     EMPTY,        DUMMY,       0,        &(W_amw_yd),    0,     0,             4},
  {S_amwsup,      UECSSHOWDATA,   U_perl,     EMPTY,        DUMMY,       0,        &(W_amw_all),   0,     0,             4},
  {S_EC2meL,      UECSINPUTDATA,  U_EC2meL,   EMPTY,        DUMMY,       0,        &(W_EC2meL),    0,     1000,          1},
  {S_nutriEC,     UECSINPUTDATA,  U_EC,       EMPTY,        DUMMY,       0,        &(W_nutriEC),   0,     10000,         3},
  {S_waterEC,     UECSINPUTDATA,  U_EC,       EMPTY,        DUMMY,       0,        &(W_waterEC),   0,     10000,         3}
};

//////////////////////////////////
// UserCCM setting
//////////////////////////////////

//define CCMID for identify
//CCMID_dummy must put on last
enum { 
  CCMID_cnd,        CCMID_rcM,      CCMID_rcA,
  CCMID_TdNtInt,    CCMID_TdWtInt,
  CCMID_TdMEInt,    CCMID_TdLQInt,  CCMID_SrInt,
  CCMID_WRadiation, CCMID_Time,
  CCMID_dummy
};
const int U_MAX_CCM = CCMID_dummy;
UECSCCM U_ccmList[U_MAX_CCM];

const char ccmNameCnd[] PROGMEM = "機器状態";
const char ccmTypeCnd[] PROGMEM = "cnd.cNM";
const char ccmUnitCnd[] PROGMEM = "";

const char ccmNamercA[] PROGMEM = "遠隔制御";
const char ccmTypercA[] PROGMEM = "rcA.cNM";

const char ccmNamercM[] PROGMEM = "手動操作";
const char ccmTypercM[] PROGMEM = "rcM.cNM";
const char ccmUnitP[] PROGMEM = "%";

const char ccmNameTdNtInt[] PROGMEM = "日給肥量";
const char ccmTypeTdNtInt[] PROGMEM = "TdNtInt.cNM";
const char ccmUnitTdNtInt[] PROGMEM = "cm3 plant-1";

const char ccmNameTdWtInt[] PROGMEM = "日給水量";
const char ccmTypeTdWtInt[] PROGMEM = "TdWtInt.cNM";

const char ccmNameTdMEInt[] PROGMEM = "日給ME量";
const char ccmTypeTdMEInt[] PROGMEM = "TdMEInt.cNM";
const char ccmUnitTdMEInt[] PROGMEM = "me plant-1";

const char ccmNameTdLQInt[] PROGMEM = "日給液量";
const char ccmTypeTdLQInt[] PROGMEM = "TdLQInt.cNM";

const char ccmNameSrInt[] PROGMEM = "日射積算";
const char ccmTypeSrInt[] PROGMEM = "SrInt.cNM";
const char ccmUnitSrInt[] PROGMEM = "kJ";

const char ccmNameWRad[] PROGMEM = "日射フラックス";
const char ccmTypeWRad[] PROGMEM = "WRadiation.mOC";
const char ccmUnitRad[] PROGMEM = "kW m-2";

const char ccmNameTime[] PROGMEM = "時刻";
const char ccmTypeTime[] PROGMEM = "Time";
const char ccmUnitTime[] PROGMEM = "時分";

//------------------------------------------------------
//UARDECS初期化用関数
//主にCCMの作成とMACアドレスの設定を行う
//------------------------------------------------------
void UserInit(){

 //Set ccm list
 UECSsetCCM(true,  CCMID_cnd,        ccmNameCnd,       ccmTypeCnd,      ccmUnitCnd,      30, 0, A_1S_0 );
 UECSsetCCM(false, CCMID_rcA,        ccmNamercA,       ccmTypercA,      ccmUnitP,        30, 0, A_1M_0 );
 UECSsetCCM(false, CCMID_rcM,        ccmNamercM,       ccmTypercM,      ccmUnitP,        30, 0, A_1S_0 );
 UECSsetCCM(true,  CCMID_TdNtInt,    ccmNameTdNtInt,   ccmTypeTdNtInt,  ccmUnitTdNtInt,  30, 1, A_1M_0 );
 UECSsetCCM(true,  CCMID_TdWtInt,    ccmNameTdWtInt,   ccmTypeTdWtInt,  ccmUnitTdNtInt,  30, 1, A_1M_0 );
 UECSsetCCM(true,  CCMID_TdMEInt,    ccmNameTdMEInt,   ccmTypeTdMEInt,  ccmUnitTdMEInt,  30, 4, A_1M_0 );
 UECSsetCCM(true,  CCMID_TdLQInt,    ccmNameTdLQInt,   ccmTypeTdLQInt,  ccmUnitTdNtInt,  30, 1, A_1M_0 );
 UECSsetCCM(true,  CCMID_SrInt,      ccmNameSrInt,     ccmTypeSrInt,    ccmUnitSrInt,    30, 3, A_1M_0 );
 UECSsetCCM(false, CCMID_WRadiation, ccmNameWRad,      ccmTypeWRad,     ccmUnitRad,      30, 3, A_1M_0 );
 UECSsetCCM(false, CCMID_Time,       ccmNameTime,      ccmTypeTime,     ccmUnitTime,     30, 0, A_1M_0 );


 //必ずシールドに貼ってあるアドレスに書き換えて使用して下さい
 U_orgAttribute.mac[0] = mac0;
 U_orgAttribute.mac[1] = mac1;
 U_orgAttribute.mac[2] = mac2;
 U_orgAttribute.mac[3] = mac3;
 U_orgAttribute.mac[4] = mac4;
 U_orgAttribute.mac[5] = mac5;
}

//---------------------------------------------------------
//起動直後に１回呼び出される関数。
//様々な初期化処理を記述できる。
//この関数内ではUECSsetup()関数を呼び出さなくてはならない。
//必要に応じて処理を記述してもかまわない。
//---------------------------------------------------------
void setup(){

  //制御シールドの制御信号線の初期化
  pinMode(in_out, OUTPUT) ;
  digitalWrite(in_out, LOW) ;
  for(i = 0; i < 4; i++) {
    pinMode(port[i], OUTPUT) ;
    digitalWrite(port[i], LOW) ;
  }
  pinMode(ratch, OUTPUT) ;
  digitalWrite(ratch, HIGH) ;
  delayMicroseconds(1) ; 
  digitalWrite(ratch, LOW) ;
  pinMode(wdt, OUTPUT) ;
  
  digitalWrite(wdt, HIGH) ; // ウオッチドッグクリア信号を初期値にする
  clearWdt() ;
  clearDout() ; // 出力をリセット

  // I2C初期化
  Wire.begin();
  
  // RTC初期化 付いていても、いなくても、とりあえずリセット
  // ここでハングアップしないので
  Wire.beginTransmission(RTC_ADRS);
  Wire.write(0xE0);
  Wire.write(0x20);  //24hourMode
  Wire.write(0x00);  // PON Clear
  Wire.endTransmission();
  delay(1);

  UECSsetup();
  
  // W_mode変数の異常でリセットの条件判断
  if((W_mode<0)||(W_mode>2)) {
    W_modeset = 4 ; //全値リセットに設定
    initializeSetpoints();
    }

  //時計設定により時刻初期値
  if(W_setdtf == 2) {
    // 外部なので時刻リセット
    v_time = W_time_in = 0 ; // 時刻0時0分
    minutes = 0 ; // 分刻0分
  }
  if(W_setdtf == 0) {
    // 内部なので
    // リアルタイムクロックから時刻読み込み
    RTCread() ;
    // RTCの値が変ならばリセット
    if((v_time > 2359)||(v_time < 0)) {
      v_time = 0 ;
      v_date = 170101 ;
      RTCset() ;
    }
  }
  pminutes = minutes ; // 一回前の分を同値にクリア
  
  //自動給肥給水残秒数リセット
  rNutSec = 0 ;
  rWtSec = 0 ;
  //1分間分の積算変数クリア
  W_srint_pr = 0 ;
  //前回給肥時刻を現在時刻にセット(0にすると起動時にほぼ給肥されてしまう)
  W_pnrtime = v_time ;
  // 秒カウンタをリセット
  Csec = 0 ;
  //積算給肥量計算用作業変数クリア
  intsupply = intsupplyw = 0 ;
}

//------------------------------------------------------
// EPROM関係の制御設定値等をクリアする。
// W_modeの変数値がおかしい場合もリセット
//------------------------------------------------------
void initializeSetpoints() {
  if(W_modeset == 4) {
    W_supply = 0 ; // 培養液供給
    W_mode = 3 ; // 給肥モード 下記強制動作
    W_modeset = 1 ; // 給肥モード設定 強制停止
    W_ntconst = 100 ; // 給肥定数 10.0 ml
    W_onesp = 120 ; // 給肥時間 120秒
    W_nf_ratio = 80 ; // 時刻給液濃度 80%
    W_na_ratio = 20 ; // 自動給液濃度 20%
    W_ntspsrint = 2000 ; // 給肥積算日射
    W_ntfcspint = 240 ; // 強制給肥間隔
    W_fctime1 = 2400 ; // 強制給肥時刻1
    W_fctime2 = 2400 ; // 強制給肥時刻2
    W_fctime3 = 2400 ; // 強制給肥時刻3
    W_fctime4 = 2400 ; // 強制給肥時刻4
    W_fctime5 = 2400 ; // 強制給肥時刻5
    W_setdtf = 2 ; // 日時 外部使用
    W_useoutsrf = 1 ; // 外部日射使用
    W_sr_in = 0 ; // 内部日射量
    W_srint_td = 0 ; // 本日積算日射
    W_pnrtime = v_time ; // 前回給肥時刻
    W_srint_pr = 0 ; // 現在積算日射
    W_amnt_td = 0 ; // 本日給肥量
    W_amnt_yd = 0 ; // 昨日給肥量
    W_amnt_all = 0 ; // 総給肥量
    W_amw_td = 0 ; // 本日給水量
    W_amw_yd = 0 ; // 昨日給水量
    W_amw_all = 0 ; // 総給水量
    W_EC2meL = 100 ; // 1dS/m = 10.0 me/L
    W_waterEC = 150 ; // 原水EC
    W_nutriEC = 1200 ; // 原水EC
  }
}

//------------------------------------------------------
// 手動操作が指定されたときに所定の動作を実施する
//------------------------------------------------------
void manualOperation() {
  if((W_modeset == 0)&&(W_mode == 3)) {
    // 自動になった場合
    W_mode = 0 ; // 強制モード
    nutrientOFF() ;
  }
  if(W_modeset == 1) {
    // 強制給肥停止の場合
    W_mode = 3 ; // 強制モード
    nutrientOFF() ; // waterOFF()も兼ねる
  }
  if(W_modeset == 2) {
    // 強制給肥の場合
    W_mode = 3 ; // 強制モード
    nutrientON() ;
  }
  if(W_modeset == 3) {
    // 強制給水の場合
    W_mode = 3 ; // 強制モード
    waterON() ;
  }
}

//------------------------------------------------------
// ネットからのリモート操作 rcM.cMNのCCM
// 有効のとき、-1: リモート操作解除、
//             0 給肥停止
//             1 ～ 100　給肥開始
//             -2 ～ -100 給水開始
//------------------------------------------------------
void rcMOperation() {  
  if(W_mode <= 2) {
    // auto,rcA,rcMの時のみ処理
    if(!(U_ccmList[CCMID_rcM].validity)) {
      if(W_mode == 2) {
        W_mode = 0 ; //AUTOモード
        nutrientOFF() ;
      }
    }
    // リモートCCMが有効だったら処理
    else {
      // rcMが-1でリモート操作モードだったら解除
      if(U_ccmList[CCMID_rcM].value == -1) {
        if(W_mode == 2) {
          W_mode = 0 ; //AUTOモード
          nutrientOFF() ;
        }
      }
      else {
        // rcMが0-100%の意味ある値であれば
        if((U_ccmList[CCMID_rcM].value >= 0)&&(U_ccmList[CCMID_rcM].value <= 100)) {
          W_mode = 2 ; // リモート操作モードに
          // それぞれの値で処理
          switch(U_ccmList[CCMID_rcM].value) {
            case 0:
              nutrientOFF() ;
              break ;
            default: // それ以外、つまり1-100のとき
              nutrientON() ;
              break ;
          }
        }
        // rcMが-2から-100%の意味ある値であれば 給水
        if((U_ccmList[CCMID_rcM].value >= -100)&&(U_ccmList[CCMID_rcM].value <= -2)) {
          W_mode = 2 ; // リモート操作モードに
          waterON() ;
        }
      }
    }
  }
}

//------------------------------------------------------
// ネットからのリモート制御 rcA.cMNのCCM
// 有効のとき、-1: リモート制御解除、
//             0 給肥停止
//             1-100　給肥開始
//------------------------------------------------------
void rcAOperation() {
  if(W_mode <= 1) {
    // auto,rcAの時のみ処理
    if(!(U_ccmList[CCMID_rcA].validity)) {
      if(W_mode == 1) {
        W_mode = 0 ; //AUTOモード
        nutrientOFF() ;
      }
    }
    // リモートCCMが有効だったら処理
    else {
      // rcAが-1でリモート制御モードだったら解除
      if(U_ccmList[CCMID_rcA].value == -1) {
        if(W_mode == 1) {
          W_mode = 0 ; //AUTOモード
          nutrientOFF() ;
        }
      }
      else {
        // rcAが0-100%の意味ある値であれば
        if((U_ccmList[CCMID_rcA].value >= 0)&&(U_ccmList[CCMID_rcA].value <= 100)) {
          W_mode = 1 ; // リモート制御モードに
          // それぞれの値で処理
          switch(U_ccmList[CCMID_rcA].value) {
            case 0:
              nutrientOFF() ;
              break ;
            default: // それ以外、つまり1-100のとき
              nutrientON() ;
              break ;
          }
        }
        // rcMが-2から-100%の意味ある値であれば 給水
        if((U_ccmList[CCMID_rcA].value >= -100)&&(U_ccmList[CCMID_rcA].value <= -2)) {
          W_mode = 1 ; // リモート操作モードに
          waterON() ;
        }
      }
    }
  }
}

//------------------------------------------------------
// 自動給肥の処理プログラム
//------------------------------------------------------
// 1分毎に呼びだして条件をチェックし給肥時間をrNutSecに加算
void autoCheck() { 
  if(W_mode == 0) {
    // 自律モードであれば
    // 時刻での給肥
    if(v_time == W_fctime1) {
      rNutSec += W_onesp * W_nf_ratio / 100 ;
      rWtSec += W_onesp * (100 - W_nf_ratio) / 100 ;
      W_srint_pr = 0 ; // 積算日射クリア
    }
    if(v_time == W_fctime2) {
      rNutSec += W_onesp * W_nf_ratio / 100 ;
      rWtSec += W_onesp * (100 - W_nf_ratio) / 100 ;
      W_srint_pr = 0 ; // 積算日射クリア
    }
    if(v_time == W_fctime3) {
      rNutSec += W_onesp * W_nf_ratio / 100 ;
      rWtSec += W_onesp * (100 - W_nf_ratio) / 100 ;
      W_srint_pr = 0 ; // 積算日射クリア
    }
    if(v_time == W_fctime4) {
      rNutSec += W_onesp * W_nf_ratio / 100 ;
      rWtSec += W_onesp * (100 - W_nf_ratio) / 100 ;
      W_srint_pr = 0 ; // 積算日射クリア
    }
    if(v_time == W_fctime5) {
      rNutSec += W_onesp * W_nf_ratio / 100 ;
      rWtSec += W_onesp * (100 - W_nf_ratio) / 100 ;
      W_srint_pr = 0 ; // 積算日射クリア
    }
    //強制給肥間隔超過による強制給肥
    if((W_ntfcspint > 0)&&(W_sr_in > 1)) { // 設定値が0の時または日射量が0.001kW以下のときには行わない
      if(W_pnrtime == 0) W_pnrtime = v_time ; // 午前0時から初めて日射量が0でなくなった時には、
      //ここから強制給肥経過時間を計算し、夜明け時に強制給肥しないために、前回給肥時刻を夜明けの時刻に設定する
      if(elapsMin(v_time, W_pnrtime) >= W_ntfcspint) {
        rNutSec += W_onesp * W_na_ratio / 100 ;
        rWtSec += W_onesp * (100 - W_na_ratio) / 100 ;
        W_srint_pr = 0 ; // 積算日射クリア
      }
    }
    //積算日射超過による自動給肥
    if(W_srint_pr >= (W_ntspsrint * 1000)) {
      rNutSec += W_onesp * W_na_ratio / 100 ;
      rWtSec += W_onesp * (100 - W_na_ratio) / 100 ;
      W_srint_pr = 0 ;
    }
  }
}

// 1秒毎に呼びだしてrNutSecで給肥し、1減算
void autoControl(){
  if(W_mode == 0) {
    //自律動作･給水を優先
    if(rWtSec > 0) { // 給水が必要なら給水
      waterON() ;
      W_pnrtime = v_time ;
      if(rWtSec > 0) rWtSec-- ; // 水供給残時間の減算
    }
    else {
      if(rNutSec > 0) { // 給水してないとき、給肥が必要なら給肥
        nutrientON() ;
        W_pnrtime = v_time ;
        if(rNutSec > 0) rNutSec-- ; // 養液供給残時間の減算
      }
      else {
      nutrientOFF() ;
      }
    }
  } 
}  
  
//------------------------------------------------------
// 日射量培養液午前0時関係計算
//------------------------------------------------------
void calc0am(){
  //日射量関係
  W_srint_td = 0 ;
  W_srint_pr = 0 ;
  
  //培養液関係
  W_amnt_yd = W_amnt_td ;
  W_amnt_td = 0 ;
  W_amw_yd = W_amw_td ;
  W_amw_td = 0 ;
  
  //前回給肥時刻もリセットする
  W_pnrtime = 0 ;
}  

//------------------------------------------------------
// CCMに値をセットする。
// 毎秒チェック
//------------------------------------------------------
void setCCM(){
  // cndに制御モード値をセット
  U_ccmList[CCMID_cnd].value = W_mode ;
  // 本日給肥量セット
  U_ccmList[CCMID_TdNtInt].value = W_amnt_td ;
  // 本日給水量セット
  U_ccmList[CCMID_TdWtInt].value = W_amw_td ;
  // 本日給液量セット
  U_ccmList[CCMID_TdLQInt].value = W_amnt_td + W_amw_td ;
  // 本日給ME量セット
  // ver3.2  1dS/mで10.0 me/LのときがW_EC2meL = 100 なので、
  // 計算値にW_EC2meLを掛けて100で割る
  U_ccmList[CCMID_TdMEInt].value 
   = (W_amnt_td * W_nutriEC + W_amw_td * W_waterEC) / 100 
     * W_EC2meL / 100 ;
  // 本日日射積算値セット
  U_ccmList[CCMID_SrInt].value = W_srint_td ;
}

//---------------------------------------------------------
//Webページから入力が行われ各種値を設定し不揮発性メモリに値を保存後、
//ページの再描画を行う前に以下の関数が呼び出される。
//---------------------------------------------------------
void OnWebFormRecieved(){
  if(W_setdtf == 3) { // 外部設定ならば、時刻をセットしてから外部読み込み
    v_time = W_time_in ;
    W_setdtf = 2 ;
    W_pnrtime = v_time ; // 時刻変更後強制給肥しないために前回給肥時刻にもセット
    Csec = 0 ; // 秒もリセット
  }

  if(W_setdtf == 2) { // 外部ならば、とりあえず外部時刻を読んでみようとする
    if(U_ccmList[CCMID_Time].validity) { // Time CCMが有効ならば
      W_time_in = v_time = U_ccmList[CCMID_Time].value / 100 ; // 読み込みセット
    }
  }

  if(W_setdtf == 0) { // 内部時計使用ならば
    // リアルタイムクロックから時刻読み込み
    RTCread() ;
    W_time_in = v_time ;
  }
  
  if(W_setdtf == 1) { // 内部時計の設定ならば
    // 内部時刻変更
    v_time = W_time_in ;
    v_date = 170101 ;
    RTCset() ;
    W_setdtf = 0 ;
    W_pnrtime = v_time ; // 時刻変更後強制給肥しないために前回給肥時刻にもセット
    Csec = 0 ; // 秒もリセット
  }

  // モード変更に基づく処理　飛び先に条件判断あり
  initializeSetpoints(); //リセット処理
  manualOperation(); // Web強制操作処理

}

//---------------------------------------------------------
//１秒に１回呼び出される関数
//---------------------------------------------------------
void UserEverySecond(){

  //ウオッチドッグタイマークリア
  clearWdt() ;
  
  //時刻関係処理
  pminutes = minutes ; // 1分の変わり目の判断変数の更新
  // 現在時刻読み込みか計算
  if(W_setdtf == 2) { // 外部ならば、とりあえず外部時刻を読んでみようとする
    Csec++ ; // 秒カウンタUP
    if(U_ccmList[CCMID_Time].validity) { // Time CCMが有効ならば
      W_time_in = v_time = U_ccmList[CCMID_Time].value / 100 ; // 読み込みセット
      if(Csec > 59) Csec = 0 ;
    }
    else { // CCM無効のときには不正確だが自律的に経時
      if(Csec > 59) {
        Csec = 0 ;
        if(v_time > 2359) { v_time = 0 ; }
        else {
          if((v_time % 100) == 59) { v_time = v_time + 100 - 59 ; }
          else v_time++ ;
        }
        W_time_in = v_time ;
      }
    }
    minutes = v_time % 100 ; // minutesに分刻をセット
  }
  if(W_setdtf == 0) { // 内部ならば、
    RTCread() ; //v_timeに時刻読み出し
    W_time_in = v_time ;
    Csec++ ; // RTCの時にはminutesがRTCから読み込み時に自動セットされるが
             // 外部と内部の切替時に不安定にならないように秒カウンタを処理
    if(Csec > 59) Csec = 0 ;
  }

  if(minutes != pminutes) {
    //////////////////////////////////////////////////////
    // 1分毎の処理
    // void UserEveryMinute()は誤差がある可能性があるので
    // 使用しない(1回給肥しなかったりすると大変!)
    //////////////////////////////////////////////////////
    Csec = 0 ; // 秒カウンタ強制リセット
    
    // 0:00の処理
    if(v_time == 0) calc0am() ; // 日射量培養液午前0時関係計算
    autoCheck() ; // 1分毎の自動給肥条件チェック    
    W_amnt_td += (intsupply / 60) ; // 本日給肥量の計算
    W_amnt_all += (intsupply / 60) ; // 総給肥量の計算
    W_amw_td += (intsupplyw / 60) ; // 本日給水量の計算
    W_amw_all += (intsupplyw / 60) ; // 総給水量の計算
    intsupply = intsupplyw =  0 ; // 1分間積算値クリア
  }
  
  /////////////////////
  // 以下は1秒毎の処理
  /////////////////////
  autoControl() ; // 自律自動制御処理
  rcMOperation() ; // リモート操作処理
  rcAOperation() ; // リモート制御処理
  
  if(W_useoutsrf == 1) { // 外部日射を使用の設定
    if(U_ccmList[CCMID_WRadiation].validity) {
      // 外部日射を使用の設定で、CCMが有効ならば、外部値を積算
      W_sr_in = U_ccmList[CCMID_WRadiation].value ;
      W_srint_pr += (float)U_ccmList[CCMID_WRadiation].value ;
      W_srint_td += (float)U_ccmList[CCMID_WRadiation].value ;
    }
    else { // CCMが無効ならば、0}
      W_sr_in = 0 ;
    }
  }
  else {
    // それ以外は内部日射を積算
    W_sr_in = (signed long)SRmeasure() ; // センサ日射量測定
    W_srint_pr += W_sr_in ;
    W_srint_td += W_sr_in ;
  }
  
  //供給培養液・水量計算・ME計算
  if(W_supply == 1) {
    intsupply += W_ntconst ;
  }
  if(W_supply == 2) {
    intsupplyw += W_ntconst ;
  }
  
  setCCM() ; // CCMの値計算
}

//---------------------------------------------------------
//１分に１回呼び出される関数
//---------------------------------------------------------
void UserEveryMinute(){
}

//---------------------------------------------------------
//メインループ
//システムのタイマカウント，httpサーバーの処理，
//UDP16520番ポートと16529番ポートの通信文をチェックした後，呼び出さされる関数。
//呼び出される頻度が高いため，重い処理を記述しないこと。
//---------------------------------------------------------
void UserEveryLoop(){
}

//---------------------------------------------------------
//setup()実行後に呼び出されるメインループ
//この関数内ではUECSloop()関数を呼び出さなくてはならない。
//UserEveryLoop()に似ているがネットワーク関係の処理を行う前に呼び出される。
//必要に応じて処理を記述してもかまわない。
//呼び出される頻度が高いため,重い処理を記述しないこと。
//---------------------------------------------------------
void loop(){
  UECSloop();
}
